#ifndef CSEARCH_H
#define CSEARCH_H

// Common definitions
#define MAX_SPM_SIZE (1 << 20)

#include <stdio.h>
#include "stdint.h"

typedef struct Node_struct {
	unsigned	size;
	unsigned	parent;
	unsigned	edge_weight;
	unsigned	region;
} Node;

typedef struct Back_Edge_struct {
    unsigned    header, footer, edge_weight;
} Back_Edge;

typedef struct Problem_struct {
	unsigned 	spm_size;
	char		mode;
	Node *		nodes;
	unsigned	num_nodes;
	unsigned	quantum_size;
	unsigned	fixed_cost;
    Back_Edge * back_edges;
    unsigned    num_back_edges;
} Problem;

// Common functions
uint64_t validate(Problem * p);
void print_partition(Problem * p, FILE * fd);
Problem * read_problem(FILE * fd);
void write_problem(Problem * p, FILE * fd);
void delete_problem(Problem * p);

// Search modules
uint64_t exhaustive_search(Problem * p);
uint64_t extended_lukes(Problem * p);

// "main"
uint64_t csearch(const char * problem, char algorithm, int enable_printf);

#endif

